@@ -1241,7 +1241,7 @@ void idbm_recinfo_config(recinfo_t *info, FILE *f)
 			*(name+i) = *nl; i++; nl++;
 		}
 		if (!*nl) {
-			log_warning("config file line %d do not has value",
+			log_warning("Config file line %d does not have value",
 			       line_number);
 			continue;
 		}
@@ -1249,7 +1249,7 @@ void idbm_recinfo_config(recinfo_t *info, FILE *f)
 		/* skip after-name traling spaces */
 		while (*nl && isspace(c = *nl)) nl++;
 		if (*nl && *nl != '=') {
-			log_warning("config file line %d has not '=' sepa",
+			log_warning("Config file line %d does not have '=' separator",
 			       line_number);
 			continue;
 		}
@@ -1258,7 +1258,7 @@ void idbm_recinfo_config(recinfo_t *info, FILE *f)
 		/* skip after-sepa traling spaces */
 		while (*nl && isspace(c = *nl)) nl++;
 		if (!*nl) {
-			log_warning("config file line %d do not has value",
+			log_warning("Config file line %d does not have value",
 			       line_number);
 			continue;
 		}
@@ -1835,7 +1835,7 @@ int idbm_print_all_discovery(int info_level)
  * @port: rec's port
  *
  * This will run fn over all recs with the {targetname,tpgt,ip,port}
- * id. It does not iterate over the ifaces setup in /etc/iscsi/ifaces.
+ * id. It does not iterate over the ifaces setup in the iface DB directory.
  *
  * fn should return -1 if it skipped the rec, an ISCSI_ERR error code if
  * the operation failed or 0 if fn was run successfully.
@@ -3031,9 +3031,9 @@ free_info:
 int idbm_init(idbm_get_config_file_fn *fn)
 {
 	/* make sure root db dir is there */
-	if (access(ISCSI_CONFIG_ROOT, F_OK) != 0) {
-		if (mkdir(ISCSI_CONFIG_ROOT, 0770) != 0) {
-			log_error("Could not make %s %d", ISCSI_CONFIG_ROOT,
+	if (access(ISCSI_DB_ROOT, F_OK) != 0) {
+		if (mkdir(ISCSI_DB_ROOT, 0770) != 0) {
+			log_error("Could not make %s %d", ISCSI_DB_ROOT,
 				   errno);
 			return errno;
 		}
