@@ -243,7 +243,13 @@ static int login_session(struct node_rec *rec)
 	 * trying to connect. Retry with exponential backoff, start from 50 ms.
 	 */
 	for (msec = 50; msec <= 15000; msec <<= 1) {
-		rc = iscsid_exec_req(&req, &rsp, 0, ISCSID_REQ_TIMEOUT);
+		/*
+		 * Once our event loop is up then we want to wait for the login
+		 * response. Either it logs in, we hit the login retries count,
+		 * or this program crashes, so there no need for the response
+		 * timeout.
+		 */
+		rc = iscsid_exec_req(&req, &rsp, 0, -1);
 		if (rc == 0) {
 			return rc;
 		} else if (rc == ISCSI_ERR_ISCSID_NOTCONN) {
@@ -347,7 +353,7 @@ int main(int argc, char *argv[])
 	struct boot_context *context, boot_context;
 	struct sigaction sa_old;
 	struct sigaction sa_new;
-	struct user_param *param;
+	struct user_param *param, *tmp_param;
 	int control_fd, mgmt_ipc_fd, err;
 	pid_t pid;
 
@@ -542,7 +548,7 @@ int main(int argc, char *argv[])
 	mgmt_ipc_close(mgmt_ipc_fd);
 	free_initiator();
 	sysfs_cleanup();
-	list_for_each_entry(param, &user_params, list) {
+	list_for_each_entry_safe(param, tmp_param, &user_params, list) {
 		list_del(&param->list);
 		idbm_free_user_param(param);
 	}
