@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <pwd.h>
 #include <sys/un.h>
+#include <string.h>
 
 #include "iscsid.h"
 #include "idbm.h"
@@ -410,8 +411,12 @@ mgmt_ipc_write_rsp(queue_task_t *qtask, int err)
 	}
 
 	qtask->rsp.err = err;
-	if (write(qtask->mgmt_ipc_fd, &qtask->rsp, sizeof(qtask->rsp)) < 0)
-		log_error("IPC qtask write failed: %s", strerror(errno));
+	if (write(qtask->mgmt_ipc_fd, &qtask->rsp, sizeof(qtask->rsp)) < 0) {
+		if (qtask->conn && qtask->conn->session)
+			conn_error(qtask->conn, "IPC qtask write failed: %s", strerror(errno));
+		else
+			log_error("IPC qtask write failed: %s", strerror(errno));
+	}
 	mgmt_ipc_destroy_queue_task(qtask);
 }
 
@@ -452,8 +457,11 @@ mgmt_ipc_read_req(queue_task_t *qtask)
 		/* Remember the allocated pointer in the
 		 * qtask - it will be freed by write_rsp.
 		 * Note: we allocate one byte in excess
-		 * so we can append a NUL byte. */
+		 * so we can append a NULL byte. */
 		qtask->payload = malloc(req->payload_len + 1);
+		if (!qtask->payload)
+			return -ENOMEM;
+
 		rc = mgmt_ipc_read_data(qtask->mgmt_ipc_fd,
 				qtask->payload,
 				req->payload_len);
